package org.objectweb.fractal.bf.connectors.ws;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashMap;
import java.util.logging.Logger;

import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.frontend.ClientProxyFactoryBean;
import org.objectweb.fractal.bf.StubGenerationException;

/**
 * Handle calls to remote web-services.
 */
public class WsInvocationHandler implements InvocationHandler {

	Logger log = Logger.getLogger(WsInvocationHandler.class
			.getCanonicalName());

	private Object client;

	public WsInvocationHandler(WsBindHints bindHints, Class<?> clazz) {

		if (bindHints == null) {
			throw new StubGenerationException("WsBindHInts can't be null");
		}

		log
				.finest("Class name for to use creating the WsInvocationHandler: "
						+ clazz.getName());

		if (clazz.getName().endsWith("Soap")) {
			// this block of code is for pre-generated stubs
			String canonicalName = clazz.getCanonicalName();
			String simpleName = clazz.getSimpleName();
			log.finest("Reference class simple name: " + canonicalName);
			String soapEnabledClassName = clazz.getCanonicalName().substring(0,
					canonicalName.length() - 4);
			log.finest("soap enabled class name: " + soapEnabledClassName);
			try {
				Class<?> factoryClass = Class.forName(soapEnabledClassName);
				Object factoryInstance = factoryClass.newInstance();
				// XXX hard-coding the name-scheme used by wsdl2java and similar
				// tools
				Method factoryMethod = factoryInstance.getClass().getMethod(
						"get" + simpleName, new Class[] {});
				client = factoryMethod.invoke(factoryInstance, new Object[] {});
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				log.warning("Could not find class :" + soapEnabledClassName
						+ " in the classpath");
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}

		} else {
			// use dynamic generation with cxf proxy factories
			final ClientProxyFactoryBean clientProxyfactoryBean = new ClientProxyFactoryBean();
			clientProxyfactoryBean.setServiceClass(clazz);
			String address = bindHints.getAddress();
			log.finer("Address: " + address);
			clientProxyfactoryBean.setAddress(address);
			final AegisDatabinding aegisDatabinding = new AegisDatabinding();

			// with cxf 2.0.5
			HashMap<String, Object> props = new HashMap<String, Object>();
			props.put(AegisDatabinding.WRITE_XSI_TYPE_KEY, true);
			clientProxyfactoryBean.setProperties(props);
			// with cxf 2.1
			// AegisContext aegisCtx = new AegisContext();
			// aegisCtx.setReadXsiTypes(true);
			// aegisDatabinding.setAegisContext(aegisCtx);
			clientProxyfactoryBean.setDataBinding(aegisDatabinding);
			client = clientProxyfactoryBean.create();
		}
	}

	/**
	 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object,
	 *      java.lang.reflect.Method, java.lang.Object[])
	 */
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {

		String methodName = method.getName();

		log.finer("Invoking method " + methodName + " with args: "
				+ Arrays.deepToString(args) + " on webservice.");

		Object result = null;

		final Method remoteMethod = client.getClass().getMethod(methodName,
				method.getParameterTypes());
		log.finer("Will invoke method: " + remoteMethod
				+ " on cxf client proxy: " + client);
		result = remoteMethod.invoke(client, args);

		log.finer("Result retured by proxy: " + result);

		return result;
	}
}
